@@ -743,3 +743,152 @@ void ContentTest::testFreezing()
     delete msg;
 }
 
+void ContentTest::testContentTypeMimetype_data()
+{
+    QTest::addColumn<QByteArray>("data");
+    QTest::addColumn<QByteArray>("mimetype");
+    QTest::addColumn<int>("contentCount");
+    QTest::addColumn<QList<QByteArray> >("contentMimeType");
+    QByteArray data =
+        "From: Nathaniel Borenstein <nsb@bellcore.com>\n"
+        "To: Ned Freed <ned@innosoft.com>\n"
+        "Date: Sun, 21 Mar 1993 23:56:48 -0800 (PST)\n"
+        "Subject: Sample message\n"
+        "MIME-Version: 1.0\n"
+        "Content-type: multipart/mixed; boundary=\"simple boundary\"\n"
+        "\n"
+        "This is the preamble.  It is to be ignored, though it\n"
+        "is a handy place for composition agents to include an\n"
+        "explanatory note to non-MIME conformant readers.\n"
+        "\n"
+        "--simple boundary\n"
+        "\n"
+        "This is implicitly typed plain US-ASCII text.\n"
+        "It does NOT end with a linebreak.\n"
+        "--simple boundary\n"
+        "Content-type: text/plain; charset=us-ascii\n"
+        "\n"
+        "This is explicitly typed plain US-ASCII text.\n"
+        "It DOES end with a linebreak.\n"
+        "\n"
+        "--simple boundary--\n"
+        "\n"
+        "This is the epilogue.  It is also to be ignored.\n";
+
+    QList<QByteArray> contentMimes = { "", "text/plain"};
+    QTest::newRow("multipart") << data << QByteArrayLiteral("multipart/mixed") << 2 << contentMimes;
+
+    data =
+            "From: Montel Laurent <null@kde.org>\n"
+            "To: kde-commits@kde.org\n"
+            "Content-Type: text/plain; charset=\"utf-8\"\n"
+            "MIME-Version: 1.0\n"
+            "Content-Transfer-Encoding: quoted-printable\n"
+            "Subject: [libksieve] src/ksieveui: coding style\n"
+            "Date: Tue, 30 May 2017 19:25:59 +0000\n"
+            "\n"
+            "Git commit 5410a2b3b6eef29ba32d0ac5e363fd38a56f535b by Montel Laurent.\n"
+            "Committed on 30/05/2017 at 19:25.\n";
+    QTest::newRow("text/plain") << data << QByteArrayLiteral("text/plain") << 0 << QList<QByteArray>();
+
+    data =
+            "From: Montel Laurent <null@kde.org>\n"
+            "To: kde-commits@kde.org\n"
+            "Content-Type: %22%22%22%22%22%22%22%22%22%22%22%22%22%22%22%22=?windows-1252?q?ap%22/pdf;\n"
+            "name=\"file.pdf\"\n"
+            "MIME-Version: 1.0\n"
+            "Content-Transfer-Encoding: quoted-printable\n"
+            "Subject: [libksieve] src/ksieveui: coding style\n"
+            "Date: Tue, 30 May 2017 19:25:59 +0000\n"
+            "\n"
+            "Git commit 5410a2b3b6eef29ba32d0ac5e363fd38a56f535b by Montel Laurent.\n"
+            "Committed on 30/05/2017 at 19:25.\n";
+    QTest::newRow("broken") << data << QByteArrayLiteral("text/plain") << 0 << QList<QByteArray>();
+
+    data =
+            "From: Montel Laurent <null@kde.org>\n"
+            "To: kde-commits@kde.org\n"
+            "Content-Type: application/pdf;\n"
+            "name=\"file.pdf\"\n"
+            "MIME-Version: 1.0\n"
+            "Content-Transfer-Encoding: quoted-printable\n"
+            "Subject: [libksieve] src/ksieveui: coding style\n"
+            "Date: Tue, 30 May 2017 19:25:59 +0000\n"
+            "\n"
+            "Git commit 5410a2b3b6eef29ba32d0ac5e363fd38a56f535b by Montel Laurent.\n"
+            "Committed on 30/05/2017 at 19:25.\n";
+    QTest::newRow("not broken") << data << QByteArrayLiteral("application/pdf") << 0 << QList<QByteArray>();
+
+    data =
+            "From kde@kde.org Fri Jan 29 19:19:26 2010\n"
+            "Return-Path: <kde@kde.org>\n"
+            "Delivered-To: kde@kde.org\n"
+            "Received: from localhost (localhost [127.0.0.1])\n"
+            "        by mail.kde.org (Postfix) with ESMTP id 8A6FF2B23D\n"
+            "        for <kde@kde.org>; Fri, 29 Jan 2010 19:19:27 +0100 (CET)\n"
+            "From: KDE <kde@kde.org>\n"
+            "To: kde@kde.org\n"
+            "Subject: Attachment test Outlook\n"
+            "Date: Fri, 29 Jan 2010 19:19:26 +0100\n"
+            "User-Agent: KMail/1.13.0 (Linux/2.6.32-gentoo-r1; KDE/4.3.95; x86_64; ; )\n"
+            "MIME-Version: 1.0\n"
+            "Content-Type: Multipart/Mixed;\n"
+            "  boundary=\"Boundary-00=_uayYLfn6pjD7iFW\"\n"
+            "Message-Id: <201001291919.26518.kde@kde.org>\n"
+            "X-Length: 10467\n"
+            "X-UID: 17\n"
+            "\n"
+            "--Boundary-00=_uayYLfn6pjD7iFW\n"
+            "Content-Type: Text/Plain;\n"
+            "  charset=\"us-ascii\"\n"
+            "Content-Transfer-Encoding: 7bit\n"
+            "\n"
+            "Attachment test Outlook\n"
+            "\n"
+            "--Boundary-00=_uayYLfn6pjD7iFW\n"
+            "Content-Type: text/x-patch;\n"
+            "  charset=\"UTF-8\";\n"
+            "  name*=UTF-8''%C3%A5%2Ediff\n"
+            "Content-Transfer-Encoding: 7bit\n"
+            "Content-Disposition: attachment;\n"
+            "  filename=\"=?iso-8859-1?q?=E5=2Ediff?=\"\n"
+            "\n"
+            "Index: kmime/kmime_headers_p.h\n"
+            "===================================================================\n"
+            "--- kmime/kmime_headers_p.h     (revision 1068282)\n"
+            "+++ kmime/kmime_headers_p.h     (working copy)\n"
+            "@@ -170,6 +170,7 @@\n"
+            "     int lines;\n"
+            " };\n"
+            "\n"
+            "+kmime_mk_empty_private( ContentID, Generics::SingleIdent )\n"
+            " }\n"
+            "\n"
+            " }\n"
+            "\n"
+            "--Boundary-00=_uayYLfn6pjD7iFW--";
+    contentMimes = { "text/plain", "text/x-patch"};
+    QTest::newRow("example1") << data << QByteArrayLiteral("multipart/mixed") << 2 << contentMimes;
+}
+
+
+void ContentTest::testContentTypeMimetype()
+{
+    QFETCH(QByteArray, data);
+    QFETCH(QByteArray, mimetype);
+    QFETCH(int, contentCount);
+    QFETCH(QList<QByteArray> , contentMimeType);
+
+    // test parsing
+    Message *msg = new Message();
+    msg->setContent(data);
+    msg->parse();
+    QEXPECT_FAIL("broken", "Problem with content type", Continue);
+    QCOMPARE(msg->contentType(false)->mimeType(), mimetype);
+    QCOMPARE(msg->contents().count(), contentCount);
+    for (int i = 0; i < msg->contents().count(); ++i) {
+        QVERIFY(msg->contents().at(i)->contentType(false));
+        QCOMPARE(contentMimeType.count(), contentCount);
+        QCOMPARE(msg->contents().at(i)->contentType(false)->mimeType(), contentMimeType.at(i));
+    }
+}
